// BACKEND_MEM.v
// All memory enables are active low
// BACKEND_MEM.vhd
// Revision 1, 04/20/2005
// BACKEMD_MEM is a structure of memory instantiations to form the backend memory of Actel's Core1553BRT which can be accessed by  
//              a Core8051 as host processor.
// BACKEND_MEM ports must be connected to the system as below
// CLK: MIL-STD_1553 system clock
// Ports which their name starts with 'CPU': Connect to the corresponding Core8051 memory interface ports. For example CPU_memdatao
//                                           connects to the memdatao port of Core8051
// Ports which their name starts with 'RT': Connect to the corresponding Core1553BRT memory inteface ports. For example RT_MEMADDR
//                                          connects to the MEMADDR port of Core1553BRT
//
// In this implementation, the MSB of MEMADDR port of Core1553BRT is unsed and can be left floating in the system.

 
module BACKEND_MEM (CLK, CPU_memdatao, CPU_memdatai, CPU_memwr, CPU_memrd, CPU_memaddr,
                    RT_MEMDOUT, RT_MEMDIN, RT_MEMWRn, RT_MEMRDn, RT_MEMADDR);
                    
input CLK, CPU_memrd, CPU_memwr, RT_MEMWRn, RT_MEMRDn ; 
input [10:0] CPU_memaddr;
input [7:0] CPU_memdatao;
input [15:0] RT_MEMDOUT;
output[7:0] CPU_memdatai;
input [9:0] RT_MEMADDR; //MSB of 1553BRT MEMADDR (MEMADDR[10]) output is unused and can be left floating.
output [15:0] RT_MEMDIN;

reg [7:0] CPU_memdatai;
wire [15:0] bottom_read;
wire [1:0] top_wen;

TOP_MEM TOP_HALF (.WDATA({CPU_memdatao,CPU_memdatao}), .WADDR(CPU_memaddr[10:1]), .WENB(top_wen), .RADDR(RT_MEMADDR), 
                  .RDATA(RT_MEMDIN), .RENB(RT_MEMRDn), .RCLK(CLK), .WCLK(CLK));

BOTTOM_MEM BOTTOM_HALF (.WDATA(RT_MEMDOUT), .WADDR(RT_MEMADDR), .RDATA(bottom_read), .RADDR(CPU_memaddr[10:1]), 
                        .RCLK(CLK), .WCLK(CLK), .WENB(RT_MEMWRn), .RENB(!CPU_memrd));

always @ (bottom_read or CPU_memaddr[0])
begin
if (CPU_memaddr[0] == 1'b0)
CPU_memdatai <= bottom_read[7:0];
else 
CPU_memdatai <= bottom_read[15:8];
end

assign top_wen[1] = !(CPU_memwr && CPU_memaddr[0]); //memory enables active-low and cpu enables active_hi
assign top_wen[0] = !(!CPU_memwr && CPU_memaddr[0]);  //memory enables active-low and cpu enables active_hi

endmodule